﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Windows.Devices.Bluetooth.Advertisement;

namespace WpfApp1
{
    /// <summary>
    /// MainWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class MainWindow : Window
    {
        BluetoothLEAdvertisementWatcher advWatcher;
        StreamWriter csvWriter;

        public MainWindow()
        {
            InitializeComponent();

            this.advWatcher = new BluetoothLEAdvertisementWatcher();
            this.advWatcher.SignalStrengthFilter.SamplingInterval = TimeSpan.FromMilliseconds(5000);
            this.advWatcher.Received += this.Watcher_Received;

            this.configureView();
        }

        private void configureView()
        {
            this.scanIntervalComboBox.Items.Add("1秒");
            this.scanIntervalComboBox.Items.Add("5秒");
            this.scanIntervalComboBox.Items.Add("10秒");
            // ※10秒以上はなぜか不安定になる。。
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="args"></param>
        private async void Watcher_Received(BluetoothLEAdvertisementWatcher sender, BluetoothLEAdvertisementReceivedEventArgs args)
        {
            await this.Dispatcher.InvokeAsync(() => {
                // 参考：Windows10デバイスでiBeaconの全データを取得する方法　http://sonic.blue/it/605
                iBeacon bcon = new iBeacon(args);
                int rssiUnderLimit = Int32.Parse(this.rssiLowerLimitTextBox.Text);
                if (bcon.UUID != null && bcon.Rssi >= rssiUnderLimit)
                {
                    DateTime dt = DateTime.Now;
                    string uuid = bcon.UUID.Replace("-", "");

                    string resultForDisplay;
                    resultForDisplay = dt.ToString("yyyy/MM/dd HH:mm:ss") + ": ";
                    resultForDisplay += string.Format("{0}, ", uuid);
                    //resultForDisplay += string.Format("major:{0},", bcon.Major.ToString("D"));
                    resultForDisplay += string.Format("{0}, ", bcon.Minor.ToString("D"));
                    //resultForDisplay += string.Format("measuredPower:{0},", bcon.MeasuredPower.ToString("D"));
                    resultForDisplay += string.Format("RSSI:{0},", bcon.Rssi.ToString("D"));
                    //resultForDisplay += string.Format("accuracy:{0},", bcon.Accuracy.ToString("F6"));
                    //resultForDisplay += string.Format("proximity:{0}", bcon.Proximity);

                    this.csvWriter.WriteLine("{0},{1},{2},{3}", dt.ToString("yyyy/MM/dd HH:mm:ss"), uuid, bcon.Minor.ToString("D"), bcon.Rssi.ToString("D"));
                    this.resultTextBox.Text = this.resultTextBox.Text + resultForDisplay + "\r\n";
                }
            });
        }

        private void scanButton_Click(object sender, RoutedEventArgs e)
        {
            if (this.advWatcher.Status == BluetoothLEAdvertisementWatcherStatus.Started)
            {
                this.ScanButton.IsEnabled = false;
                this.ScanButton.Content = "終了中...";
                this.csvWriter.Close();
                this.advWatcher.Stop();
                Thread.Sleep(1000);
                this.ScanButton.Content = "スキャン開始";
                this.ScanButton.IsEnabled = true;
            } else
            {
                this.advWatcher.Start();
                this.ScanButton.Content = "スキャン終了";
                string desktopDirectoryPath = Environment.GetFolderPath(Environment.SpecialFolder.DesktopDirectory);
                this.csvWriter = new StreamWriter(desktopDirectoryPath + @"\beacons.csv", true, Encoding.UTF8);
            }
        }

        private void clearDisplayButton_Click(object sender, RoutedEventArgs e)
        {
            this.resultTextBox.Text = "";
        }

        private void scanIntervalComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            string contentString = (string)scanIntervalComboBox.SelectedItem;
            if (contentString == null) return;

            this.advWatcher.Stop();
            Thread.Sleep(1000); // Stopした後に直ぐに設定を行うとうまく反映されないので少し待つ
            if (contentString == "1秒")
            {
                this.advWatcher.SignalStrengthFilter.SamplingInterval = TimeSpan.FromMilliseconds(1 * 1000);
            }
            else if (contentString == "5秒")
            {
                this.advWatcher.SignalStrengthFilter.SamplingInterval = TimeSpan.FromMilliseconds(5 * 1000);
            }
            else if (contentString == "10秒")
            {
                this.advWatcher.SignalStrengthFilter.SamplingInterval = TimeSpan.FromMilliseconds(10 * 1000);
            }
            this.advWatcher.Start();
        }
    }
}
